/*******************************************************************************
 * Copyright 2015 - 2020 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe;

import java.io.IOException;

import de.frankmuenster.mahoe.utils.BankToAdd;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.Utils;
import de.frankmuenster.mahoe.web.AbstractSeleniumWebSite;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.plugin.Version;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Main class of the plugin
 *
 * @author Frank Mnster
 */
public class MahoeMain extends AbstractPlugin {

    /**
     * Standardkonstruktor
     */
    public MahoeMain() {

        super();
    }

    /**
     *
     */
    @Override
    public void init() {

        // Einmal alle Provider laden, damit die BLZ eingetragen werde
        this.loadProviders();
    }

    /**
     * Task to be done during shutdown of the application
     */
    @Override
    public void shutDown() {
        // anything in super?
        super.shutDown();

        // close selenium window
        AbstractSeleniumWebSite.shutDown();

    }

    /**
     * @see AbstractPlugin#update(Version)
     */
    @Override
    public void update(final Version oldVersion) throws ApplicationException {

        Settings.setOldVersion(oldVersion);
    }

    /**
     * Lade die Job Provider damit die untersttzte BLZ Liste gefllt wird.
     */
    private void loadProviders() {

        Class<?>[] classes;
        try {
            classes = Utils.getClasses("de.frankmuenster.mahoe"); //$NON-NLS-1$
            for (final Class<?> clazz : classes) {
                if (clazz.isAnnotationPresent(BankToAdd.class)) {
                    // Read bank information from annotation
                    final BankToAdd blzAnnotation = clazz.getAnnotation(BankToAdd.class);
                    final String blz = blzAnnotation.blz();
                    final String name = blzAnnotation.name();
                    final String walletPrefix = blzAnnotation.walletPrefix();
                    // Add to supported bank (BLZ) list
                    MahoeSynchronizeBackend.addBlzSupported(blz, name);
                    MahoeSynchronizeBackend.addWalletPrefix(walletPrefix, name);
                    Logger.info(I18N.tr("MahoeMain.loadBLZ", name, blz)); //$NON-NLS-1$
                }
            }
        } catch (final IOException e) {
            Logger.error(I18N.tr("MahoeMain.loadingClassesFailed", e.getLocalizedMessage()), e); //$NON-NLS-1$
        }
    }

}
