/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.aktien.assistant;

import java.rmi.RemoteException;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.mahoe.assistant.AbstractCreateAccountAssistant;
import de.frankmuenster.mahoe.banks.aktien.Aktien;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.hbci.HBCI;
import de.willuhn.jameica.hbci.rmi.HBCIDBService;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Aktien Konto-Anlagen-Assistent
 *
 * @author Frank Mnster
 */
public final class AktienCreateAccountAssistent
        extends AbstractCreateAccountAssistant<AktienCreateInfo, AktienCreateAccountDialog> {

    /**
     * Standardkonstruktor
     */
    public AktienCreateAccountAssistent() {

        super();
    }

    /**
     * @see de.frankmuenster.mahoe.assistant.AbstractCreateAccountAssistant#getDialog()
     */
    @Override
    public AktienCreateAccountDialog getDialog() {

        return new AktienCreateAccountDialog();
    }

    /**
     * @throws RemoteException
     * @throws ApplicationException
     * @see de.frankmuenster.mahoe.assistant.AbstractCreateAccountAssistant#setBankSpecificMetaData(de.frankmuenster.mahoe.assistant.CreateInfo,
     *      de.willuhn.jameica.hbci.rmi.Konto)
     */
    @Override
    public void setBankSpecificMetaData(final AktienCreateInfo info, final Konto konto)
            throws RemoteException, ApplicationException {

        // Nothing to do here
    }

    @Override
    protected void setBankSpecificAccountParameter(final Konto account, final AktienCreateInfo info)
            throws RemoteException {

        account.setBLZ(Aktien.BLZ);
        account.setKontonummer(this.getKto());
        account.setKundennummer("Aktien"); //$NON-NLS-1$

        account.setUnterkonto(StringUtils.EMPTY);

        // is this still needed?
        account.setAccountType(KontoType.WERTPAPIERDEPOT.getValue());
    }

    /**
     * Liefert die nchst freie niedrigere Aktien KTO Nummer (angefangen bei Aktien.KTO)
     *
     * @return die Kontonummer
     */
    private String getKto() {

        HBCIDBService dbService = null;
        Long newNumber = null;
        try {
            dbService = (HBCIDBService) Application.getServiceFactory().lookup(HBCI.class, "database"); //$NON-NLS-1$

            newNumber = Long.parseLong(Aktien.KTO);
            final DBIterator<Konto> kontos = dbService.createList(Konto.class);
            while (kontos.hasNext()) {
                final Konto kto = kontos.next();
                if (Aktien.BLZ.equals(kto.getBLZ())) {
                    final Long toCheck = Long.parseLong(kto.getKontonummer());
                    if (toCheck.equals(newNumber)) {
                        newNumber--;
                        kontos.begin();
                    }
                }
            }
        } catch (final Exception e) { // NOPMD
            Logger.error("KTO-Nr. konnte nicht ermittelt werden.", e); //$NON-NLS-1$
        }
        return newNumber == null ? null : Long.toString(newNumber);
    }

}
