/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.aktien.jobs.web;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import com.gargoylesoftware.htmlunit.WebClient;

import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.web.AbstractWebSite;
import de.frankmuenster.mahoe.web.MahoeKontoauszugPDF;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.util.ApplicationException;

/**
 * @author Frank Mnster
 */
public final class AktienWebSite extends AbstractWebSite<WebClient, AktienWebAccountInformation, AktienBankMessages> {

    /**
     * Our Instance
     */
    private static AktienWebSite instance;

    /**
     * @param konto
     * @return die Webseiten Instanz
     * @throws WebSiteException
     */
    public static AktienWebSite getInstance(final Konto konto) throws WebSiteException {

        if (AktienWebSite.instance == null) {
            AktienWebSite.instance = new AktienWebSite(konto);
        }
        return AktienWebSite.instance;
    }

    /**
     * Standardkonstruktor
     *
     * @param konto
     * @throws WebSiteException
     */
    private AktienWebSite(final Konto konto) throws WebSiteException {
        super(konto);
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#createSepaTransfer(de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung)
     */
    @Override
    public void createSepaTransfer(final AuslandsUeberweisung aUeberweisung)
            throws WebSiteException, ApplicationException {

        // brauchen wir nicht
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#getAccountInformation()
     */
    @Override
    public AktienWebAccountInformation getAccountInformation() throws WebSiteException, ApplicationException {

        return this.accountInfo;
    }

    @Override
    public List<MahoeKontoauszugPDF> getAccountSheets(final AktienWebAccountInformation accountInfo)
            throws WebSiteException {
        throw new WebSiteException(I18N.tr("AbstractWebSite.pdfDownloadNotSupported"), //$NON-NLS-1$
                WebSiteExceptionTypes.OPERATION_NOT_SUPPORTED);

    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#getBookedTransactions()
     */
    @Override
    public List<Umsatz> getBookedTransactions() throws WebSiteException, ApplicationException {

        return new ArrayList<>();
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#getPreBookedTransactions()
     */
    @Override
    public List<Umsatz> getPreBookedTransactions() throws WebSiteException, ApplicationException {

        return new ArrayList<>();
    }

    /**
     * @see de.frankmuenster.mahoe.web.AbstractWebSite#getWebClient()
     */
    @Override
    public WebClient getWebClient() {

        return this.webClient;
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#isStillLoggedIn()
     */
    @Override
    public boolean isStillLoggedIn() throws WebSiteException {

        return true;
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#login(char[], java.lang.String[])
     */
    @Override
    public void login(final char[] password, final String... loginParams) throws WebSiteException {

        // brauchen wir nicht
    }

    /**
     * @see de.frankmuenster.mahoe.web.WebSite#logoff()
     */
    @Override
    public void logoff() throws WebSiteException {

        // brauchen wir nicht
    }

    /**
     * @see de.frankmuenster.mahoe.web.AbstractWebSite#getStartdate()
     */
    @Override
    protected Calendar getStartdate() {

        return Calendar.getInstance();
    }
}
