/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.bcee.web; // NOPMD

import java.io.IOException;
import java.rmi.RemoteException;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import com.gargoylesoftware.htmlunit.FailingHttpStatusCodeException;
import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.html.DomElement;
import com.gargoylesoftware.htmlunit.html.HtmlButton;
import com.gargoylesoftware.htmlunit.html.HtmlDivision;
import com.gargoylesoftware.htmlunit.html.HtmlElement;
import com.gargoylesoftware.htmlunit.html.HtmlInput;
import com.gargoylesoftware.htmlunit.html.HtmlPage;
import com.gargoylesoftware.htmlunit.html.HtmlSpan;
import com.gargoylesoftware.htmlunit.html.HtmlTable;
import com.gargoylesoftware.htmlunit.html.HtmlTextInput;

import de.frankmuenster.mahoe.banks.bcee.Bcee;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.Utils;
import de.frankmuenster.mahoe.web.AbstractWebSite;
import de.frankmuenster.mahoe.web.MahoeKontoauszugPDF;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * BCEE Bank Website.
 *
 * @author Frank Mnster
 */
public final class BceeWebSite extends AbstractWebSite<WebClient, BceeWebAccountInformation, BceeBankMessages> {

    /** Unsere BceeWebSite instance. */
    private static BceeWebSite webSite;

    /** Die LuxTrust User ID. */
    private final String luxTrustUserId;

    /** TransactionImpl Objekt. */
    private final BceeWebSiteTransactionsImpl transImpl;

    /**
     * Liefert die Instance der WebSeite.
     *
     * @param konto
     *            the konto
     * @return die WebSeite
     * @throws WebSiteException
     *             the web site exception
     * @throws ApplicationException
     *             the application exception
     */
    public static BceeWebSite getInstance(final Konto konto) throws WebSiteException, ApplicationException {

        String luxTrustUser = null;
        try {
            luxTrustUser = konto.getKundennummer();
        } catch (final RemoteException e) {
            final String msg = I18N.tr("BceeWebSite.loginMissing"); // NOPMD //$NON-NLS-1$
            Logger.error(msg, e);
            throw new ApplicationException(msg, e);
        }
        if (BceeWebSite.webSite == null) { // NOPMD
            BceeWebSite.webSite = new BceeWebSite(konto, luxTrustUser);
        } else {
            // Ist die LuxTrustUserId dieselbe
            if (BceeWebSite.webSite.luxTrustUserId.equals(luxTrustUser)) {
                // Aktualisiere Startdatum der Umsatzabfrage und lsche
                // Umsatzliste
                final Date lastTransacDate = Utils.getLastUmsatzDate(konto, BceeWebSite.webSite.getStartdate());
                BceeWebSite.webSite.accountInfo.setLastTransactionDate(lastTransacDate);
                BceeWebSite.webSite.transImpl.clear();
            } else {
                BceeWebSite.webSite = new BceeWebSite(konto, luxTrustUser);
            }
        }
        return BceeWebSite.webSite;
    }

    /**
     * Erstellt ein neues Webseiten-Objekt zur Kontaktaufnahme mit der BCEE.
     *
     * @param konto
     *            the konto
     * @param luxTrustUserId
     *            the lux trust user id
     * @throws WebSiteException
     *             the web site exception
     */
    private BceeWebSite(final Konto konto, final String luxTrustUserId) throws WebSiteException {

        super(konto);
        this.luxTrustUserId = luxTrustUserId;
        this.webClient = BceeWebSiteUtils.getWebClient();
        this.transImpl = new BceeWebSiteTransactionsImpl();
    }

    /**
     * Creates the sepa transfer.
     *
     * @param aUeberweisung
     *            the a ueberweisung
     * @throws WebSiteException
     *             the web site exception
     * @throws ApplicationException
     *             the application exception
     * @see de.frankmuenster.mahoe.web.WebSite#createSepaTransfer(de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung)
     */
    @Override
    public void createSepaTransfer(final AuslandsUeberweisung aUeberweisung)
            throws WebSiteException, ApplicationException {

        // not supported
    }

    /**
     * Gets the account information.
     *
     * @return the account information
     * @throws WebSiteException
     *             the web site exception
     * @throws ApplicationException
     *             the application exception
     * @see de.frankmuenster.mahoe.web.WebSite#getAccountInformation()
     */
    @Override
    public BceeWebAccountInformation getAccountInformation() throws WebSiteException, ApplicationException {

        // ffne Konto
        HtmlPage page = BceeWebSiteUtils.openAccount(this.webClient, this.accountInfo);
        // Saldo lesen
        final DomElement accountDiv = page.getElementById("content-own-LIQUIDITT"); //$NON-NLS-1$
        final HtmlSpan saldoAmount = accountDiv.getFirstByXPath("div/div[3]/div/span"); //$NON-NLS-1$
        final String saldo = saldoAmount.getTextContent();
        this.accountInfo.setSaldo(Utils.convertStringToBigDecimal(saldo));
        // Nun die KTO-Infos ffnen --> Kredit und Zinssatz
        final HtmlElement actInfo = (HtmlElement) page.getElementById("accountInformationButton"); //$NON-NLS-1$
        try {
            final int scriptCount1 = page.getWebClient().waitForBackgroundJavaScript(1000);
            Logger.debug("scriptCount1: " + scriptCount1); //$NON-NLS-1$
            page = actInfo.click();
            int scriptCount2 = page.getWebClient().waitForBackgroundJavaScriptStartingBefore(1000);
            Logger.debug("scriptCount2: " + scriptCount2); //$NON-NLS-1$
            int maxLoopCounter = 10;
            // Max weitere 10 Sek. warten
            do {
                maxLoopCounter--;
                scriptCount2 = page.getWebClient().waitForBackgroundJavaScript(1000);
                Logger.debug("scriptCount2: " + scriptCount2); //$NON-NLS-1$
            } while (maxLoopCounter > 0 && scriptCount2 > scriptCount1);
            final HtmlDivision actInfoDiv = (HtmlDivision) page.getElementById("accountInformationModal"); //$NON-NLS-1$
            if (actInfoDiv.asText().contains("Kontoinformationen")) { //$NON-NLS-1$
                final HtmlTable actInfoTbl = actInfoDiv.getFirstByXPath("div/div/div[2]/table"); //$NON-NLS-1$
                // Keditlimit auslesen
                final String credit = actInfoTbl.getRow(4).getCell(1).asText();
                this.accountInfo.setCreditLimit(Utils.convertStringToDouble(credit));
                // Sollzins
                String interest = actInfoTbl.getRow(5).getCell(1).asText();
                interest = interest.replaceAll("%", StringUtils.EMPTY).replaceAll(" ", StringUtils.EMPTY); //$NON-NLS-1$//$NON-NLS-2$
                this.accountInfo.setInterestRate(Utils.convertStringToDouble(interest));
            }
        } catch (final IOException e) {
            Logger.error(I18N.tr("BceeWebSite.errorOpeningAccountDetails"), e); //$NON-NLS-1$
        }
        return this.accountInfo;
    }

    @Override
    public List<MahoeKontoauszugPDF> getAccountSheets(final BceeWebAccountInformation accountInfo)
            throws WebSiteException {

        throw new WebSiteException(I18N.tr("AbstractWebSite.pdfDownloadNotSupported"), //$NON-NLS-1$
                WebSiteExceptionTypes.OPERATION_NOT_SUPPORTED);

    }

    /**
     * Gets the booked transactions.
     *
     * @return the booked transactions
     * @throws WebSiteException
     *             the web site exception
     * @throws ApplicationException
     *             the application exception
     * @see de.frankmuenster.mahoe.web.WebSite#getBookedTransactions()
     */
    @Override
    public List<Umsatz> getBookedTransactions() throws WebSiteException, ApplicationException {

        return this.transImpl.getBookedTransactions(this.webClient, this.accountInfo);
    }

    /**
     * Gets the pre booked transactions.
     *
     * @return the pre booked transactions
     * @throws WebSiteException
     *             the web site exception
     * @throws ApplicationException
     *             the application exception
     * @see de.frankmuenster.mahoe.web.WebSite#getPreBookedTransactions()
     */
    @Override
    public List<Umsatz> getPreBookedTransactions() throws WebSiteException, ApplicationException {

        return this.transImpl.getPreBookTransactions(this.webClient, this.accountInfo);
    }

    /**
     * Gets the web client.
     *
     * @return the web client
     * @see de.frankmuenster.mahoe.web.AbstractWebSite#getWebClient()
     */
    @Override
    public WebClient getWebClient() {

        return this.webClient;
    }

    /**
     * Checks if is still logged in.
     *
     * @return true, if is still logged in
     * @throws WebSiteException
     *             the web site exception
     * @see de.frankmuenster.mahoe.web.WebSite#isStillLoggedIn()
     */
    @Override
    public boolean isStillLoggedIn() throws WebSiteException {

        boolean isStillLoggedin = false;
        try {
            final HtmlPage page = this.webClient.getPage(Bcee.URL_HOME);
            Logger.debug(page.asText());
            if (page.asText().contains("Meine persnlichen Konten")) { //$NON-NLS-1$
                isStillLoggedin = true;
            }
        } catch (FailingHttpStatusCodeException | IOException e) {
            Logger.error(I18N.tr("BceeWebSite.connectionCheckFailed"), e); //$NON-NLS-1$
        }
        return isStillLoggedin;
    }

    /**
     * Login.
     *
     * @param password
     *            the password
     * @param loginParams
     *            the login params
     * @throws WebSiteException
     *             the web site exception
     * @see de.frankmuenster.mahoe.web.WebSite#login(char[], java.lang.String[])
     */
    @Override
    public void login(final char[] password, final String... loginParams) throws WebSiteException {

        Logger.debug(I18N.tr("BceeWebSite.startLogin") + this.luxTrustUserId); //$NON-NLS-1$
        if (loginParams.length != 1) { // NOPMD
            throw new WebSiteException(I18N.tr("BceeWebSite.loginParameterCountWrong"), //$NON-NLS-1$
                    WebSiteExceptionTypes.LOGIN_ERROR);
        }
        // Get Login Page
        HtmlPage page = this.getLoginPage();
        // Anmelden
        Logger.debug("Inspiziere Webseite und hole Formularfelder"); //$NON-NLS-1$
        final DomElement userIdInput = page.getElementById("user-id"); //$NON-NLS-1$
        final DomElement passwortInput = page.getElementById("password"); //$NON-NLS-1$
        final DomElement optInput = page.getElementById("otp"); //$NON-NLS-1$
        // Set Values
        Logger.info(I18N.tr("BceeWebSite.setLoginData") + this.luxTrustUserId); //$NON-NLS-1$
        ((HtmlTextInput) userIdInput).setValueAttribute(this.luxTrustUserId);
        try {
            ((HtmlInput) passwortInput).focus();
            for (final char c : password) {
                page = (HtmlPage) ((HtmlInput) passwortInput).type(c);
            }
            ((HtmlInput) optInput).focus();
            for (final char c : loginParams[0].toCharArray()) {
                page = (HtmlPage) ((HtmlInput) optInput).type(c);
            }
            // Authentication Button
            final HtmlButton submit = page
                    .getFirstByXPath("/html/body/div/div/div/div/fieldset/fieldset/div/button[2]"); //$NON-NLS-1$
            page = submit.click();
        } catch (final IOException e) {
            Logger.error(I18N.tr("BceeWebSite.settingDataFailed"), e); //$NON-NLS-1$
            throw new WebSiteException(I18N.tr("BceeWebSite.settingDataFailed"), //$NON-NLS-1$
                    WebSiteExceptionTypes.LOGIN_FAILED, e);
        }
        // Anmeldung berprfen
        this.checkLogin(page);
    }

    /**
     * Logoff.
     *
     * @see de.frankmuenster.mahoe.web.WebSite#logoff()
     */
    @Override
    public void logoff() {

        // TODO Auto-generated method stub
        Logger.info("Schliee Bankverbindung. (Noch nicht implementiert)"); //$NON-NLS-1$
    }

    /**
     * Gets the startdate.
     *
     * @return the startdate
     * @see de.frankmuenster.mahoe.web.AbstractWebSite#getStartdate()
     */
    @Override
    protected Calendar getStartdate() {

        // Setze StartDatum auf heute - 2 Jahre
        final Calendar startCalendar = Calendar.getInstance();
        startCalendar.add(Calendar.MONTH, -23);
        return startCalendar;
    }

    /**
     * &Uuml;berpr&uuml;pft die Loginseite, ob der Login erfolgreich war.
     *
     * @param page
     *            the page
     * @throws WebSiteException
     *             falls der Login nicht erfolgreich war.
     */
    private void checkLogin(final HtmlPage page) throws WebSiteException {

        final String pageText = page.asText();
        Logger.debug(pageText);
        // Click Result - TEST FOR ERR..
        if (pageText.contains("fen Sie die Eingabe der User ID")) { //$NON-NLS-1$
            this.throwLoginFailedException(I18N.tr("BceeWebSite.idInvalidText"), //$NON-NLS-1$
                    WebSiteExceptionTypes.LOGIN_FAILED);
        } else if (pageText.contains("fen Sie Ihr pers")) { //$NON-NLS-1$
            this.throwLoginFailedException(I18N.tr("BceeWebSite.passwordInvalid"), //$NON-NLS-1$
                    WebSiteExceptionTypes.LOGIN_FAILED);
        } else if (pageText.contains("LuxTrust Passwort sind unzutreffend")) { //$NON-NLS-1$
            this.throwLoginFailedException(I18N.tr("BceeWebSite.idOrPasswordMalformed"), //$NON-NLS-1$
                    WebSiteExceptionTypes.LOGIN_FAILED);
        } else if (pageText.contains("OTP (6 Zeichen)")) { //$NON-NLS-1$
            this.throwLoginFailedException(I18N.tr("BceeWebSite.otpInvalid"), //$NON-NLS-1$
                    WebSiteExceptionTypes.LOGIN_FAILED);
        } else if (pageText.contains("Ein paar Daten sind nicht korrekt")) { //$NON-NLS-1$
            this.throwLoginFailedException(I18N.tr("BceeWebSite.dataIncorrect"), //$NON-NLS-1$
                    WebSiteExceptionTypes.LOGIN_FAILED);
        }
        // And again test for error
        try {
            final String iban = Utils.ibanFormater(this.accountInfo.getKonto().getIban());
            if (pageText.contains(iban)) {
                Logger.info(I18N.tr("BceeWebSite.loginSuccessful")); //$NON-NLS-1$
            } else {
                this.throwLoginFailedException(I18N.tr("BceeWebSite.loginFailedunknownReason"), //$NON-NLS-1$
                        WebSiteExceptionTypes.LOGIN_FAILED);
            }
        } catch (final RemoteException e) {
            final String msg = I18N.tr("BceeWebSite.errorReadingAccountIban"); //$NON-NLS-1$
            Logger.error(msg);
            throw new WebSiteException(msg, WebSiteExceptionTypes.LOGIN_FAILED, e);
        }
    }

    /**
     * Liefert die Loginseite, nach Auswahl von LuxTrust und Token.
     *
     * @return die eigentliche Loginseite.
     * @throws WebSiteException
     *             the web site exception
     */
    private HtmlPage getLoginPage() throws WebSiteException {

        HtmlPage page = null;
        try {
            Logger.debug("\u00D6ffne login Seite"); //$NON-NLS-1$
            page = this.webClient.getPage(Bcee.URL_LOGIN);
            Logger.debug("Deutsch aus&auml;hlen"); //$NON-NLS-1$
            String javaScriptCode = "selectLanguage('de_DE');"; //$NON-NLS-1$
            page.executeJavaScript(javaScriptCode);
            Thread.sleep(1000);

            // Hole iFrame (MANTISBT-136)
            page = (HtmlPage) this.webClient.getWebWindowByName("orelyframe").getEnclosedPage(); //$NON-NLS-1$
            Logger.debug("\u00D6ffen Tokenseite"); //$NON-NLS-1$
            javaScriptCode = "chooseProduct('token')"; //$NON-NLS-1$
            page.executeJavaScript(javaScriptCode);

            Logger.debug(page.asText());
            page = (HtmlPage) this.webClient.getWebWindowByName("orelyframe").getEnclosedPage(); //$NON-NLS-1$
            Logger.debug(page.asText());

        } catch (FailingHttpStatusCodeException | IOException e) {
            Logger.debug(e.getMessage());
            Logger.error(I18N.tr("BceeWebSite.openStartPageFailedLog")); //$NON-NLS-1$
            throw new WebSiteException(I18N.tr("BceeWebSite.openStartPageFailed"), WebSiteExceptionTypes.PAGE_NOT_FOUND, //$NON-NLS-1$
                    e);
        } catch (final InterruptedException e) {
            Thread.currentThread().interrupt();
        }
        return page;
    }

    /**
     * Throw login failed exception.
     *
     * @param msg
     *            the message to log and add to the exception
     * @param type
     *            the type the exception type
     * @throws WebSiteException
     *             the web site exception
     */
    private void throwLoginFailedException(final String msg, final WebSiteExceptionTypes type) throws WebSiteException {

        Logger.error(msg);
        throw new WebSiteException(msg, type);

    }
}
