/*******************************************************************************
 * Copyright 2015 - 2020 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.bos.jobs;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import de.frankmuenster.mahoe.banks.bos.BankOfScotland;
import de.frankmuenster.mahoe.jobs.AbstractMahoeSynchronizeJobProvider;
import de.frankmuenster.mahoe.jobs.AbstractMahoeSynchronizeJobProviderKontoauszugPDF;
import de.willuhn.annotation.Lifecycle;
import de.willuhn.annotation.Lifecycle.Type;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.hbci.synchronize.jobs.SynchronizeJob;

/**
 * Implementation of the job provider to download account sheets for the Bank of
 * Scotland.
 *
 * @author Frank Mnster
 */
@Lifecycle(Type.CONTEXT)
public class BosSynchronizeJobProviderKontoauszugPDF extends AbstractMahoeSynchronizeJobProviderKontoauszugPDF {

    /**
     * Standard constructor
     */
    public BosSynchronizeJobProviderKontoauszugPDF() {

        super();
    }

    /**
     * @see {@link AbstractMahoeSynchronizeJobProvider#getJobs()}
     */
    @Override
    protected List<Class<? extends SynchronizeJob>> getJobs() {

        final List<Class<? extends SynchronizeJob>> list = new ArrayList<>(1);
        // add BosSynchronizeJobKontoauszugPDF
        list.add(BosSynchronizeJobKontoauszugPDF.class);
        return list;
    }

    /**
     * @see {@link AbstractMahoeSynchronizeJobProvider#getSupportedAccountTypes()}
     */
    @Override
    protected Set<KontoType> getSupportedAccountTypes() {

        final Set<KontoType> set = new HashSet<>(0);
        // add supported account types
        set.add(KontoType.GIRO); // not yet
        return set;
    }

    /**
     * @see {@link AbstractMahoeSynchronizeJobProvider#isBankSupported(Konto)}
     */
    @Override
    protected boolean isBankSupported(final Konto account) throws RemoteException {
        // check based on BLZ
        return BankOfScotland.BLZ.equals(account.getBLZ());
    }

}
