/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.targobank.web;

import java.util.List;
import java.util.Locale;

import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;

import de.frankmuenster.mahoe.banks.targobank.TargoBank;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.Utils;
import de.frankmuenster.mahoe.utils.WebUtils;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.WebSiteExceptionTypes;
import de.willuhn.logging.Logger;

/***
 * Statische Implemtierungen der allg. Funktionen
 *
 * @author Frank Mnster
 */
public final class TargoBankWebSiteImpl {

    /**
     * ffnet die Kartenbersichtsseite
     *
     * @param webClient
     * @param accountInfo
     * @return die Webseite mit der Kreditkartenbersicht und den vorgemerkten
     *         Umstzen.
     * @throws WebSiteException
     *             falls irgendwas schief geht
     */
    public static void openAccount(final WebDriver driver, final TargoBankWebAccountInformation accountInfo)
            throws WebSiteException {

        driver.get(TargoBank.URL_UEBERSICHT);

        if (driver.getPageSource().contains("bersicht")) { //$NON-NLS-1$
            // Last 4 digits of the card number NOT of the account !!!
            final String linkName = Utils.maskCardNummer(accountInfo.getCardNumber()) + " " //$NON-NLS-1$
                    + accountInfo.getCardCompany();
            Logger.debug("LinkText (Karetnkonto): " + linkName); //$NON-NLS-1$

            // Search through all links
            final List<WebElement> anchors = driver.findElements(By.tagName(WebUtils.HTML_TAG_ANCHOR));
            WebElement link = null;
            for (final WebElement anchor : anchors) {
                Logger.trace(anchor.getText());
                if (anchor.getText().equalsIgnoreCase(linkName)) {
                    Logger.debug("Anchor match!!!" //$NON-NLS-1$
                            + anchor.getText().toUpperCase(Locale.GERMAN));
                    link = anchor;
                    break;
                }
            }

            // link still null then card link was not found so raise an
            // expection
            if (link == null) {
                Logger.error(I18N.tr("TargoBankWebSiteImpl.cardNotFoundMsg") //$NON-NLS-1$
                        + linkName);
                throw new WebSiteException(I18N.tr("TargoBankWebSiteImpl.cardNotFound"), //$NON-NLS-1$
                        WebSiteExceptionTypes.KONTO_NOT_FOUND);
            } else {

                Logger.info(I18N.tr("TargoBankWebSiteImpl.cardFoundMsg", linkName)); //$NON-NLS-1$
                // Click on the card link
                link.click();
                // Check the result
                if (driver.getPageSource().contains("Betrag der letzten Zahlung")) { //$NON-NLS-1$
                    // Credit card details open --> return
                    Logger.debug("Details gefunden und offen. Nun zurueckgeben"); //$NON-NLS-1$

                } else {
                    // Something went wrong --> exception
                    Logger.error(I18N.tr("TargoBankWebSiteImpl.cardDetailsNotFound")); //$NON-NLS-1$
                    throw new WebSiteException(I18N.tr("TargoBankWebSiteImpl.cardDetailsNotFound"), //$NON-NLS-1$
                            WebSiteExceptionTypes.KONTO_NOT_FOUND);
                }
            }

        } else {
            Logger.error(I18N.tr("TargoBankWebSiteImpl.openCardOverviewFailed")); //$NON-NLS-1$
            throw new WebSiteException(I18N.tr("TargoBankWebSiteImpl.openCardOverviewFailed"), //$NON-NLS-1$
                    WebSiteExceptionTypes.KONTO_NOT_FOUND);
        }

    }

    /**
     * Privater Konstruktor
     */
    private TargoBankWebSiteImpl() {

        throw new AssertionError("Instantiating utility class..."); //$NON-NLS-1$
    }
}
