/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.gui.controller;

import java.rmi.RemoteException;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.widgets.Event;

import de.frankmuenster.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.mahoe.banks.bsh.SchwaebischHall;
import de.frankmuenster.mahoe.banks.bsh.assistant.BshCreateInfo;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.Utils;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * @author Frank Mnster
 */
public class BshSettingsController extends AbstractSettingsController<BshCreateInfo> implements SettingsController {

    /**
     * Soll jedes mal nach dem Passwort gefragt werden anlegen
     */
    private transient CheckboxInput alwaysAskForPW;

    /**
     * Show password before sending login request for debug purpose
     */
    private transient CheckboxInput debugShowPW;

    /**
     * Standardkonstruktor
     */
    public BshSettingsController() {

        super(SchwaebischHall.BLZ, new BshCreateInfo());
    }

    /**
     * Liefert eine Checkbox zur Auswahl, ob das Passwort jedes mal Abgefragt
     * werden soll.
     *
     * @return Ein CheckBoxInput Passwort jedes mal nachfragen.
     */
    public CheckboxInput getAlwaysAskForPW() {

        if (this.alwaysAskForPW == null) {
            Logger.debug("alwaysAskForPW: " + this.ktoInfo.isAlwaysAskForPW()); //$NON-NLS-1$
            this.alwaysAskForPW = new CheckboxInput(this.ktoInfo.isAlwaysAskForPW());
        }
        return this.alwaysAskForPW;
    }

    /**
     * Returns a check box to select if the password showed be shown each time a
     * login attempt is made.
     *
     * @return A CheckBoxInput
     */
    public CheckboxInput getDebugShowPW() {

        if (this.debugShowPW == null) {
            Logger.debug("debugShowPW: " + this.ktoInfo.isDebugShowPW()); //$NON-NLS-1$
            this.debugShowPW = new CheckboxInput(this.ktoInfo.isDebugShowPW());
        }
        return this.debugShowPW;
    }

    /**
     * @see de.frankmuenster.mahoe.gui.controller.SettingsController#speichern()
     */
    @Override
    public void speichern() {

        final Konto konto = (Konto) this.getKonto().getValue();
        this.ktoInfo.setAlwaysAskForPW((boolean) this.getAlwaysAskForPW().getValue());
        this.ktoInfo.setDebugShowPW((boolean) this.getDebugShowPW().getValue());
        try {
            // save settings
            final String alwaysAsk = Utils.getYesNoStringFromBoolean(this.ktoInfo.isAlwaysAskForPW());
            konto.setMeta(MahoeSynchronizeBackend.PW_ASK_ALWAYS, alwaysAsk);
            final String debugShow = Utils.getYesNoStringFromBoolean(this.ktoInfo.isDebugShowPW());
            konto.setMeta(MahoeSynchronizeBackend.PW_DEBUG_SHOW, debugShow);
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("SettingsController.saveSettingsFailedLog"), e); //$NON-NLS-1$
            final String msg = I18N.tr("SettingsController.saveSettingsFailedMsg"); //$NON-NLS-1$
            final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
            Application.getMessagingFactory().sendMessage(msgUser);
        }
    }

    /**
     * @see de.frankmuenster.mahoe.gui.controller.AbstractSettingsController#handleAccountChanged(org.eclipse.swt.widgets.Event)
     */
    @Override
    protected void handleAccountChanged(final Event event) {

        // Kontotyp ermitteln
        final Konto konto = (Konto) this.account.getValue();
        this.ktoInfo.setComment(StringUtils.EMPTY);
        this.ktoInfo.setDescription(StringUtils.EMPTY);
        this.ktoInfo.setKdNummer(StringUtils.EMPTY);
        this.ktoInfo.setOwner(StringUtils.EMPTY);

        try {
            // read account data or set to false
            if (konto == null) {
                this.ktoInfo.setAlwaysAskForPW(false);
                this.ktoInfo.setDebugShowPW(false);
            } else {
                this.ktoInfo.setAlwaysAskForPW(Utils.getBooleanFromYesNoString(konto
                        .getMeta(MahoeSynchronizeBackend.PW_ASK_ALWAYS, Utils.TEXT_NEIN).toUpperCase(Locale.GERMAN)));
                this.ktoInfo.setDebugShowPW(Utils.getBooleanFromYesNoString(konto
                        .getMeta(MahoeSynchronizeBackend.PW_DEBUG_SHOW, Utils.TEXT_NEIN).toUpperCase(Locale.GERMAN)));
            }
            // Now set the check box
            this.getAlwaysAskForPW().setValue(this.ktoInfo.isAlwaysAskForPW());
            this.getDebugShowPW().setValue(this.ktoInfo.isDebugShowPW());
        } catch (final RemoteException e) {
            final String msg = I18N.tr("BshSettingsController.readAccountSettingsFailed"); //$NON-NLS-1$
            Logger.error(msg, e);
            final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
            Application.getMessagingFactory().sendMessage(msgUser);
        }
    }
}
