/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.gui.controller;

import java.rmi.RemoteException;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import de.frankmuenster.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.mahoe.banks.santander.SantanderBank;
import de.frankmuenster.mahoe.banks.santander.assistant.SantanderCreateInfo;
import de.frankmuenster.mahoe.utils.DebitCardValidator;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.Utils;
import de.willuhn.jameica.gui.input.CheckboxInput;
import de.willuhn.jameica.gui.input.TextInput;
import de.willuhn.jameica.hbci.gui.filter.KontoFilter;
import de.willuhn.jameica.hbci.gui.input.KontoInput;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.hbci.synchronize.jobs.SynchronizeJobSepaUeberweisung;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * Kontroller fr die Settings View
 *
 * @author Frank Mnster
 */
@SuppressWarnings("PMD.GodClass")
public class SantanderSettingsController extends AbstractSettingsController<SantanderCreateInfo>
        implements SettingsController {

    /**
     * Kreditkarten Name/Anbieter
     */
    private transient TextInput cardNumber;

    /**
     * Ausgleichsbuchung anlegen
     */
    private transient CheckboxInput createClearing;

    /**
     * Abbuchungsungskonto der Ausgleichsbuchung
     */
    private transient KontoInput clearingAccount;

    /**
     * Erstellt eine neue Instanz des Kontrollers
     */
    public SantanderSettingsController() {

        super(SantanderBank.BLZ, new SantanderCreateInfo());
        // Show account type
        this.setDisplayAccountType(true);

        // set account information
        this.ktoInfo.setComment(StringUtils.EMPTY);
        this.ktoInfo.setDescription(StringUtils.EMPTY);
        this.ktoInfo.setKtoNummer(StringUtils.EMPTY);
        this.ktoInfo.setAccountType(null);
        this.ktoInfo.setLogin(StringUtils.EMPTY);
        this.ktoInfo.setOwner(StringUtils.EMPTY);

        this.ktoInfo.setCreateClearing(false);
        this.ktoInfo.setClearingAccount(null);
        this.ktoInfo.setOwner(StringUtils.EMPTY);
    }

    /**
     * Liefert ein TextInput f&uuml;r den Kartenname/anbieter
     *
     * @return ein TextInput f&uuml;r den Kartenname/anbieter
     */
    public TextInput getCardNumber() {

        if (this.cardNumber == null) {
            this.cardNumber = new TextInput(StringUtils.EMPTY);
            if (StringUtils.isNotEmpty(this.ktoInfo.getKtoNummer())) {
                this.cardNumber.setValue(this.ktoInfo.getKtoNummer());
            }
            // Kartenname als Kommentar
            if (StringUtils.isNotEmpty(this.ktoInfo.getOwner())) {
                this.cardNumber.setComment(this.ktoInfo.getOwner());
            } else {
                this.cardNumber.setComment(StringUtils.EMPTY);
            }
        }
        return this.cardNumber;
    }

    /**
     * Liefert ein Kontoauswahlfeld
     *
     * @return Kontoauswahlfeld
     */
    public KontoInput getClearingAccount() {

        if (this.clearingAccount == null) {
            final Konto konto = this.ktoInfo.getClearingAccount();
            try {
                final KontoFilter ktoFilter = KontoFilter.createForeign(SynchronizeJobSepaUeberweisung.class);
                this.clearingAccount = new KontoInput(konto, ktoFilter);
            } catch (final RemoteException e) {
                Logger.error(I18N.tr("SettingsController.createAccountListFailed"), e); //$NON-NLS-1$
            }
        }
        if (this.ktoInfo.isCreateClearing()) {
            this.clearingAccount.enable();
        } else {
            this.clearingAccount.disable();
        }
        return this.clearingAccount;
    }

    /**
     * Liefert eine Checkbox zur Auswahl, ob die Kreditkartenauszge gespeichert
     * werden sollen.
     *
     * @return Ein CheckBoxInput zur Auswahl, ob die Kreditkartenauszge
     *         gespeichert werden sollen.
     */
    public CheckboxInput getCreateClearing() {

        if (this.createClearing == null) {
            Logger.debug("createClearing: " + this.ktoInfo.isCreateClearing()); //$NON-NLS-1$
            this.createClearing = new CheckboxInput(this.ktoInfo.isCreateClearing());
            this.createClearing.addListener(new Listener() {

                /**
                 * @see Listener#handleEvent(Event)
                 */
                @Override
                public void handleEvent(final Event event) {

                    SantanderSettingsController.this.handleCreateTransaction();
                }
            });
        }
        return this.createClearing;
    }

    /**
     * @see de.frankmuenster.mahoe.gui.controller.SettingsController#speichern
     *      ()
     */
    @Override
    @SuppressWarnings("PMD.NPathComplexity")
    public void speichern() {

        this.ktoInfo.setStatementsMaxDaysBack((int) this.statementsMaxDaysBack.getValue());
        final Boolean testsOK = this.validateData();
        if (testsOK) {
            final Konto konto = (Konto) this.getKonto().getValue();
            try {
                // Daten speichern
                if (this.ktoInfo.getAccountType().equals(KontoType.GIRO)) {
                    konto.setMeta(MahoeSynchronizeBackend.CARD_NUMBER, StringUtils.EMPTY);
                    konto.setMeta(MahoeSynchronizeBackend.CARD_COMPANY, StringUtils.EMPTY);
                    // Ausgleichbuchung
                    konto.setMeta(MahoeSynchronizeBackend.CREATE_CLEARING, Utils.TEXT_NEIN);
                    konto.setMeta(MahoeSynchronizeBackend.CLEARING_ACCOUNT, StringUtils.EMPTY);
                } else {
                    konto.setMeta(MahoeSynchronizeBackend.CARD_NUMBER, this.ktoInfo.getKtoNummer());
                    // Ausgleichbuchung
                    final String myCreateClearing = Utils.getYesNoStringFromBoolean(this.ktoInfo.isCreateClearing());
                    konto.setMeta(MahoeSynchronizeBackend.CREATE_CLEARING, myCreateClearing);
                    if (this.ktoInfo.isCreateClearing()) {
                        konto.setMeta(MahoeSynchronizeBackend.CLEARING_ACCOUNT,
                                this.ktoInfo.getClearingAccount().getID());
                    } else {
                        konto.setMeta(MahoeSynchronizeBackend.CLEARING_ACCOUNT, StringUtils.EMPTY);
                    }
                }

                final String downloadPeriod = Integer.toString(this.ktoInfo.getStatementsMaxDaysBack());
                konto.setMeta(MahoeSynchronizeBackend.ACC_STATEM_MAX_DAYS_BACK, downloadPeriod);
                final String msg = I18N.tr("SettingsController.settingsSaved"); //$NON-NLS-1$
                Logger.info(msg);
                final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_INFO);
                Application.getMessagingFactory().sendMessage(msgUser);
            } catch (final RemoteException e) {
                Logger.error(I18N.tr("SettingsController.saveSettingsFailedLog"), e); //$NON-NLS-1$
                final String msg = I18N.tr("SettingsController.saveSettingsFailedMsg"); //$NON-NLS-1$
                final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
                Application.getMessagingFactory().sendMessage(msgUser);
            }
        }
    }

    /**
     * Aktualisiert die Anzeige, falls ein neues Konto ausgewhlt wurde und nur
     * dann.
     *
     * @param event
     *            Typ muss <i>SWT.Selection</i> sein, sonst wird nix gemacht.
     */
    @Override
    @SuppressWarnings("PMD.CyclomaticComplexity")
    protected void handleAccountChanged(final Event event) {

        if (event.type != SWT.Selection) {
            return;
        }
        // Kontotyp ermitteln
        final Konto konto = (Konto) this.account.getValue();
        // Not used fields
        this.ktoInfo.setComment(StringUtils.EMPTY);
        this.ktoInfo.setDescription(StringUtils.EMPTY);
        this.ktoInfo.setLogin(StringUtils.EMPTY);
        this.ktoInfo.setOwner(StringUtils.EMPTY);
        if (konto == null) {
            this.ktoInfo.setAccountType(KontoType.SONSTIGE);
            this.ktoInfo.setKtoNummer(StringUtils.EMPTY);
            this.ktoInfo.setStatementsMaxDaysBack(0);
        } else {
            try {
                // Kontoinformationen setzten
                this.ktoInfo.setKtoNummer(konto.getMeta(MahoeSynchronizeBackend.CARD_NUMBER, StringUtils.EMPTY));
                this.ktoInfo.setAccountType(KontoType.find(konto.getAccountType()));
                this.ktoInfo.setOwner(konto.getMeta(MahoeSynchronizeBackend.CARD_COMPANY, StringUtils.EMPTY));
                final int statmMaxDaysBack = Integer
                        .parseInt(konto.getMeta(MahoeSynchronizeBackend.ACC_STATEM_MAX_DAYS_BACK, "90")); //$NON-NLS-1$
                this.ktoInfo.setStatementsMaxDaysBack(statmMaxDaysBack);
                final Konto myClearingAccount = Utils
                        .getKonto(konto.getMeta(MahoeSynchronizeBackend.CLEARING_ACCOUNT, StringUtils.EMPTY));
                this.ktoInfo.setClearingAccount(myClearingAccount);
                this.ktoInfo.setCreateClearing(Utils.getBooleanFromYesNoString(konto
                        .getMeta(MahoeSynchronizeBackend.CREATE_CLEARING, Utils.TEXT_NEIN).toUpperCase(Locale.GERMAN)));
            } catch (final RemoteException e) {
                final String msg = I18N.tr("SettingsController.readAccountSettingsFailed"); //$NON-NLS-1$
                Logger.error(msg, e);
                final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
                Application.getMessagingFactory().sendMessage(msgUser);
            }
        }

        this.getKontoTyp().setValue(this.ktoInfo.getAccountType().getName());
        this.getSavePDF().setValue(this.supportsStatements());
        if (this.supportsStatements()) {
            this.getStatementMaxDaysBack().setValue(this.ktoInfo.getStatementsMaxDaysBack());
            this.getStatementMaxDaysBack().enable();
        } else {
            this.getStatementMaxDaysBack().setValue(0);
            this.getStatementMaxDaysBack().disable();
        }
        this.getCreateClearing().setValue(this.ktoInfo.isCreateClearing());
        this.getClearingAccount().setValue(this.ktoInfo.getClearingAccount());
        switch (this.ktoInfo.getAccountType()) {
        case GIRO:
            this.getCardNumber().disable();
            this.getCreateClearing().disable();
            this.handleCreateTransaction();
            break;
        case KREDITKARTE:
            this.getCardNumber().enable();
            this.getCardNumber().setValue(this.ktoInfo.getKtoNummer());
            this.getCardNumber().setComment(this.ktoInfo.getOwner());
            this.getCreateClearing().enable();
            this.handleCreateTransaction();
            break;
        default:
            this.disableAll();
            break;
        }

    }

    /**
     * Disable all credit card only relevant fields
     */
    private void disableAll() {

        this.getCardNumber().disable();
        this.getCreateClearing().disable();
        this.getClearingAccount().disable();
    }

    /**
     * Enables or disables the clearing account field based on the checkbox
     */
    private void handleCreateTransaction() {

        if ((boolean) this.getCreateClearing().getValue()) {
            Logger.debug("Enable clearingAccount Fields"); //$NON-NLS-1$
            this.getClearingAccount().enable();
        } else {
            Logger.debug("Disable clearingAccount Fields"); //$NON-NLS-1$
            this.getClearingAccount().disable();
        }
    }

    /**
     * Validiert die Eingaben.
     *
     * @return <i>true</i> wenn alle ntigen Eingaben vorhanden und korrekt
     *         sind, <i>false</i> sonst.
     */
    private Boolean validateData() {

        boolean testsOK = true;
        if (KontoType.KREDITKARTE.equals(this.ktoInfo.getAccountType())) {
            this.ktoInfo.setCreateClearing((Boolean) this.createClearing.getValue());
            this.ktoInfo.setClearingAccount((Konto) this.clearingAccount.getValue());
            this.ktoInfo.setKtoNummer((String) this.cardNumber.getValue());
            // Tests
            if (!DebitCardValidator.isValidateCardNumber(this.ktoInfo.getKtoNummer())) {
                final String msg = I18N.tr("SantanderSettingsController.wrongCreditCardNumber"); //$NON-NLS-1$
                final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
                Application.getMessagingFactory().sendMessage(msgUser);
                testsOK = false;
            }
            if (this.ktoInfo.isCreateClearing() && !(this.ktoInfo.getClearingAccount() instanceof Konto)) {
                final String msg = I18N.tr("SantanderSettingsController.provideClearingAccount"); //$NON-NLS-1$
                final StatusBarMessage msgUser = new StatusBarMessage(msg, StatusBarMessage.TYPE_ERROR);
                Application.getMessagingFactory().sendMessage(msgUser);
                testsOK = false;
            }
        }
        return testsOK;
    }
}
