
package de.frankmuenster.mahoe.gui.dialog;

import java.rmi.RemoteException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import de.frankmuenster.mahoe.MahoeMain;
import de.frankmuenster.mahoe.utils.I18N;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.dialogs.AbstractDialog;
import de.willuhn.jameica.gui.parts.ButtonArea;
import de.willuhn.jameica.gui.parts.FormTextPart;
import de.willuhn.jameica.gui.util.Container;
import de.willuhn.jameica.gui.util.LabelGroup;
import de.willuhn.jameica.gui.util.SWTUtil;
import de.willuhn.jameica.gui.util.SimpleContainer;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.system.Application;
import de.willuhn.util.ApplicationException;

/**
 * "AboutDialog..." Dialog. Basiert auf dem About dialog aus Jameica von Olaf
 * Willuhn
 *
 * @author willuhn/Frank Mnster
 * @version 1.1
 */
public class AboutDialog extends AbstractDialog<Object> {

    /**
     * Liefetr einen modalen Dialog, der mittel .open() angezeigt werden kann
     *
     * @param position
     *            die Position, an der der Dialog angezeigt werden soll.
     *            Mgliche Werte sind POSITION_MOUSE und POSITION_CENTER.
     */
    public AboutDialog(final int position) {

	super(position);

	final AbstractPlugin plugin = Application.getPluginLoader().getPlugin(MahoeMain.class);

	this.setTitle(I18N.tr("AboutDialog.about")); //$NON-NLS-1$
	this.setPanelText(I18N.tr("AboutDialog.panelText", //$NON-NLS-1$
		plugin.getManifest().getVersion().toString()));
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#getData()
     */
    @Override
    protected Object getData() {

	return null;
    }

    /**
     * @see de.willuhn.jameica.gui.dialogs.AbstractDialog#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    protected void paint(final Composite parent) throws RemoteException {

	final AbstractPlugin plugin = Application.getPluginLoader().getPlugin(MahoeMain.class);

	final Container image = new SimpleContainer(parent, false, 2);
	final GridLayout glImage = new GridLayout();
	glImage.numColumns = 2;
	image.getComposite().setLayout(glImage);
	final CLabel imageLabel = new CLabel(image.getComposite(), SWT.NONE);
	final GridData gdImage = new GridData();
	gdImage.grabExcessHorizontalSpace = true;
	gdImage.horizontalAlignment = SWT.RIGHT;
	gdImage.horizontalSpan = 2;

	imageLabel.setImage(SWTUtil.getImage("mahoe_300x220.png")); //$NON-NLS-1$
	imageLabel.setMargins(20, 10, 20, 20);

	final Container container = new LabelGroup(parent, I18N.tr("AboutDialog.versionInfo"), true); //$NON-NLS-1$

	final StringBuilder formString = new StringBuilder(300) // NOPMD
		.append("<form><p><b>") //$NON-NLS-1$
		.append(I18N.tr("AboutDialog.appTitel")) //$NON-NLS-1$
		.append("</b></p><p>") //$NON-NLS-1$
		.append(I18N.tr("AboutDialog.lisence")) //$NON-NLS-1$

		.append(" [<a href=\"https://joinup.ec.europa.eu/community/eupl/og_page/european-union-public-licence-eupl-v11\">EUPL</a>]<br/>") //$NON-NLS-1$
		.append(I18N.tr("AboutDialog.copyright")) //$NON-NLS-1$

		.append(" Frank Mnster<br />[<a href=\"mailto:jameica@frankmuenster.de\">jameica@frankmuenster.de</a>]<br/>") //$NON-NLS-1$
		.append(I18N.tr("AboutDialog.swVersion")) //$NON-NLS-1$
		.append(" ") //$NON-NLS-1$
		.append(plugin.getManifest().getVersion()).append("<br/>Build: ") //$NON-NLS-1$
		.append(plugin.getManifest().getBuildnumber()).append(" [") //$NON-NLS-1$
		.append(I18N.tr("AboutDialog.date")) //$NON-NLS-1$
		.append(" ").append(plugin.getManifest().getBuildDate()) //$NON-NLS-1$
		.append("]<br/>").append("</p></form>"); //$NON-NLS-1$ //$NON-NLS-2$
	final FormTextPart text = new FormTextPart();
	text.setText(formString.toString());

	container.addPart(text);

	final ButtonArea buttons = new ButtonArea();
	buttons.addButton(I18N.tr("AboutDialog.close"), new Action() { //$NON-NLS-1$

	    /**
	     * @see de.willuhn.jameica.gui.Action#handleAction(Object context)
	     */
	    @Override
	    public void handleAction(final Object context) throws ApplicationException {

		AboutDialog.this.close();
	    }
	}, null, true, "window-close.png"); //$NON-NLS-1$

	container.addButtonArea(buttons);

	this.setSize(SWT.DEFAULT, 520);
    }
}
