/*******************************************************************************
 * Copyright 2015 - 2020 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.statements;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

/**
 * An account statement bean with needed fields
 *
 * @author Frank Mnster
 */
public class MahoeAccountStatement {

    /**
     * The creation date
     */
    private Date createDate;

    /**
     * Name1 field
     */
    private String name1;

    /**
     * Name2 field
     */
    private String name2;

    /**
     * Name3 field
     */
    private String name3;

    /**
     * The year of the account statement. If empty the year of the creation date
     * is used.
     */
    private Integer year;

    /**
     * Comments
     */
    private String comment;

    /**
     * The year of the account statement. If empty the month of the creation
     * date is used.
     */
    private Integer number;

    /**
     * Creates a new empty statement object
     */
    public MahoeAccountStatement() {

        // Absichtliche leer
    }

    /**
     * Creates a new filled statement object
     *
     * @param createDate
     */
    public MahoeAccountStatement(final Date createDate) {

        this(createDate, null, null, null, null);
    }

    /**
     * Creates a new partially filled statement object
     *
     * @param createDate
     * @param name1
     */
    public MahoeAccountStatement(final Date createDate, final String name1) {

        this(createDate, name1, null, null, null);
    }

    /**
     * Creates a new partially filled statement object
     *
     * @param createDate
     * @param name1
     * @param name2
     */
    public MahoeAccountStatement(final Date createDate, final String name1, final String name2) {

        this(createDate, name1, name2, null, null);
    }

    /**
     * Creates a new partially filled statement object
     *
     * @param createDate
     * @param name1
     * @param name2
     * @param name3
     */
    public MahoeAccountStatement(final Date createDate, final String name1, final String name2, final String name3) {

        this(createDate, name1, name2, name3, null);
    }

    /**
     * Creates a new filled statement object
     *
     * @param createDate
     * @param name1
     * @param name2
     * @param name3
     * @param comment
     */
    public MahoeAccountStatement(final Date createDate, final String name1, final String name2, final String name3,
            final String comment) {

        this.createDate = createDate;
        this.name1 = name1;
        this.name2 = name2;
        this.name3 = name3;
        this.comment = comment;
    }

    /**
     * @return the comment
     */
    public String getComment() {

        return this.comment;
    }

    /**
     * @return the createDate
     */
    public Date getCreateDate() {

        return this.createDate;
    }

    /**
     * @return the name1
     */
    public String getName1() {

        return this.name1;
    }

    /**
     * @return the name2
     */
    public String getName2() {

        return this.name2;
    }

    /**
     * @return the name3
     */
    public String getName3() {

        return this.name3;
    }

    /**
     * Returns the number of the account statement. If not set explicitly the
     * month of the creation date is used
     *
     * @return the number of the account statement
     */
    public Integer getNumber() {

        Integer accNumber = this.number;
        if (accNumber == null) {
            final Calendar cal = new GregorianCalendar();
            cal.setTime(this.getCreateDate());
            accNumber = cal.get(Calendar.MONTH) + 1;
        }
        return accNumber;
    }

    /**
     * Returns the year of the account statement. If not set explicitly the year
     * of the creation date is used
     *
     * @return the year of the account statement
     */
    public Integer getYear() {

        Integer accYear = this.year;
        if (this.year == null) {
            final Calendar cal = new GregorianCalendar();
            cal.setTime(this.getCreateDate());
            accYear = cal.get(Calendar.YEAR);
        }
        return accYear;
    }

    /**
     * Checks is all mandatory fields are set. This is actually only the
     * createDate
     *
     * @return <i>true</i> if all mandatory fields are set, <i>false</i> else.
     */
    public boolean isComplete() {

        return this.createDate != null;
    }

    /**
     * @param comment
     *            the comment to set
     */
    public void setComment(final String comment) {

        this.comment = comment;
    }

    /**
     * @param createDate
     *            the createDate to set
     */
    public void setCreateDate(final Date createDate) {

        this.createDate = createDate;
    }

    /**
     * @param name1
     *            the name1 to set
     */
    public void setName1(final String name1) {

        this.name1 = name1;
    }

    /**
     * @param name2
     *            the name2 to set
     */
    public void setName2(final String name2) {

        this.name2 = name2;
    }

    /**
     * @param name3
     *            the name3 to set
     */
    public void setName3(final String name3) {

        this.name3 = name3;
    }

    /**
     * Sets the number of the account statement.
     *
     * @param number
     */
    public void setNumber(final Integer number) {

        this.number = number;
    }

    /**
     * Sets the year of the account statement.
     *
     * @param year
     */
    public void setYear(final Integer year) {

        this.year = year;
    }

}
